#[non_exhaustive]pub enum Error {
IncompatibleType,
Overflow,
NegativeUnsigned,
Precision,
InvalidSimpleValue,
InvalidEncoding,
InvalidUtf8,
UnexpectedEof,
LengthTooLarge,
Io(Error),
}Expand description
Errors produced during CBOR encoding, decoding, or value access.
Variants (Non-exhaustive)
This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.
IncompatibleType
Accessor called on a value of the wrong CBOR type.
Overflow
Integer does not fit in the target type.
NegativeUnsigned
Attempted to read a negative integer as an unsigned type.
Precision
Float conversion would lose precision.
InvalidSimpleValue
Simple value number is in the reserved range 24-31.
InvalidEncoding
CBOR input is not in canonical (deterministic) form.
InvalidUtf8
Text string contains invalid UTF-8.
UnexpectedEof
Input ended before a complete data item was read.
LengthTooLarge
Declared length exceeds addressable memory.
Io(Error)
Underlying I/O error.